/* Talika panel applet: Switch between open windows using icons.
 * (C) 2009-2010 Sinew Software Systems
	 *
 * Authors:  Vinod Kumar Dahiya
 *           Hemant Kumar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
	 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef TALIKA_TASK_H__
#define TALIKA_TASK_H__

#include "tasklist.h"
#include "launcher.h"

#define N_WINDOW_CONNECTIONS  3
#define N_CLASS_CONNECTIONS   2

#define WNCK_ACTIVATE_TIMEOUT 1000


/* task type
 * A window or class group or a launcher
 */
typedef enum
{
	TALIKA_TASK_CLASS_GROUP,
	TALIKA_TASK_WINDOW,
	TALIKA_TASK_LAUNCHER
} TalikaTaskType;


/* TalikaTask data*/
typedef struct _TalikaTask
{
	TalikaTaskList *tasklist;

	TalikaTaskType type;

	// task widgets
	GtkWidget *button;
	//GtkWidget *image;
	GtkWidget *menu;
	GtkWidget *action_menu;

	//if a launcher
	gchar *class_name;
	gchar *desktop_id;
	gchar *icon_path;
	GAppInfo *app_info;
	gboolean exclude;
	gint sort_order;

	//if task is a window
	WnckWindow *window;
	guint window_connections[N_WINDOW_CONNECTIONS];

	//if task is a class group
	WnckClassGroup *class_group;
	gchar *class_group_name;
	GList *windows;		
	guint class_connections[N_CLASS_CONNECTIONS];
	GtkWidget *drop_popup;
	guint drop_popup_activate;

	//task activation tracking stuff
	gboolean really_toggling;
	gboolean was_active;
	guint button_activate;

	//preview update stuff
	guint preview_update_timeout_id;

	//drag n drop stuff
	guint32 dnd_timestamp;

	//task glowing stuff
	GdkPixmap *screenshot;
	GdkPixmap *screenshot_faded;

	time_t start_needs_attention;
	gdouble glow_start_time;

	guint button_glow;

	struct _TalikaTask *launcher;
	struct _TalikaTask *attached_task;
}
TalikaTask;


/* Skipped windows */
typedef struct
{
	WnckWindow *window;
	guint tag;		//keep track of state_change signal
}
TalikaSkippedWindow;



/* TalikaTask functions */

TalikaTask* talika_task_new_from_window (TalikaTaskList * tasklist,
                                         WnckWindow * window,
                                  		 const gchar *class_group_name);

TalikaTask* talika_task_new_from_class_group (TalikaTaskList * tasklist,
                                              WnckClassGroup * class_group,
                                  			  const gchar *class_group_name);

void talika_task_free (TalikaTask * task);

gchar *talika_task_get_text (TalikaTask * task);

GdkPixbuf *talika_task_get_icon (TalikaTask * task);

void talika_task_state_changed (WnckWindow * window,
                                WnckWindowState changed_mask,
                                WnckWindowState new_state, gpointer data);

void talika_task_update_visible_state (TalikaTask * task);

void do_talika_task_update_visible_state (gpointer data, 
                                          gpointer user_data);

GdkPixbuf *talika_desaturate_icon (GdkPixbuf *pixbuf);

void 
talika_task_drag_leave (GtkWidget * widget,
                        GdkDragContext * context,
                        guint time, TalikaTask * task);

gboolean 
talika_task_drag_motion (GtkWidget * widget,
                         GdkDragContext * context,
                         gint x, gint y, 
                         guint time, 
                         TalikaTask * task);

void 
talika_task_drag_begin (GtkWidget * widget,
                        GdkDragContext * context, TalikaTask * task);

void
talika_task_drag_end (GtkWidget * widget,
                      GdkDragContext * context, TalikaTask * task);

void
talika_task_drag_data_get (GtkWidget * widget,
                           GdkDragContext * context,
                           GtkSelectionData * selection_data,
                           guint info, guint time, TalikaTask * task);

void
talika_task_drag_data_received (GtkWidget * widget,
                                GdkDragContext * context,
                                gint x,
                                gint y,
                                GtkSelectionData * data,
                                guint info,
                                guint time, TalikaTask * target_task);

void
talika_task_set_sort_order (TalikaTask *task, gint sort_order);

gint
talika_task_get_sort_order (TalikaTask *task);

#endif //TALIKA_TASK_H__
